import unittest

from generic_encryptors import AesEncryptor
from generic_encryptors.encryptors.exceptions import HmacMissMatch

class TestAesEncryptor(unittest.TestCase):

    def test_aes_encryptor_encode_decode(self):
      encryptor = AesEncryptor(key=('1' * 32).encode('ascii'))
      msg = b'123in0i1n 0in\x10\x00\x00\x00\x94\xd0z\xa6\xac\x95\xb2\xf5\x10\xfa\xd7\x9c\xf9\x9a\x8a\xd5 \x00\x00\x00\x89K\x9bGuxf8|\xf5\x93\xb6-\xbf\xab\xd0\x15\x99\xae}cu\xceWnE\xfdj7\xb3X_hc\xdao>0\x86\xcf\xecSI^\xe5<\xcfh'
      self.assertEqual(msg,
          encryptor.decode(encryptor.encode(msg)))

    def test_aes_encryptor_aborts_on_invalid_hmac_single_char_edit(self):
      encryptor = AesEncryptor(key=('1' * 32).encode('ascii'))
      msg = b'123in0i1n 0in\x10\x00\x00\x00\x94\xd0z\xa6\xac\x95\xb2\xf5\x10\xfa\xd7\x9c\xf9\x9a\x8a\xd5 \x00\x00\x00\x89K\x9bGuxf8|\xf5\x93\xb6-\xbf\xab\xd0\x15\x99\xae}cu\xceWnE\xfdj7\xb3X_hc\xdao>0\x86\xcf\xecSI^\xe5<\xcfh'
      encrypted_msg = encryptor.encode(msg)
      encrypted_msg = encrypted_msg[0:20] + b'a' + encrypted_msg[21:]
      with self.assertRaises(HmacMissMatch) as context:
        encryptor.decode(encrypted_msg)

    def test_aes_encryptor_aborts_on_invalid_hmac_appended_data(self):
      encryptor = AesEncryptor(key=('1' * 32).encode('ascii'))
      msg = b'123in0i1n 0in\x10\x00\x00\x00\x94\xd0z\xa6\xac\x95\xb2\xf5\x10\xfa\xd7\x9c\xf9\x9a\x8a\xd5 \x00\x00\x00\x89K\x9bGuxf8|\xf5\x93\xb6-\xbf\xab\xd0\x15\x99\xae}cu\xceWnE\xfdj7\xb3X_hc\xdao>0\x86\xcf\xecSI^\xe5<\xcfh'
      encrypted_msg = encryptor.encode(msg) + b'a'
      with self.assertRaises(HmacMissMatch) as context:
        encryptor.decode(encrypted_msg)

    def test_aes_encryptor_fails_with_invalid_key(self):
      encryptor1 = AesEncryptor(key=('1' * 32).encode('ascii'))
      encryptor2 = AesEncryptor(key=('2' * 32).encode('ascii'))
      msg = b'123in0i1n 0in\x10\x00\x00\x00\x94\xd0z\xa6\xac\x95\xb2\xf5\x10\xfa\xd7\x9c\xf9\x9a\x8a\xd5 \x00\x00\x00\x89K\x9bGuxf8|\xf5\x93\xb6-\xbf\xab\xd0\x15\x99\xae}cu\xceWnE\xfdj7\xb3X_hc\xdao>0\x86\xcf\xecSI^\xe5<\xcfh'
      encrypted_msg = encryptor1.encode(msg)
      with self.assertRaises(HmacMissMatch) as context:
        encryptor2.decode(encrypted_msg)


    def test_different_aes_encryptors_suceed_with_same_key(self):
      encryptor1 = AesEncryptor(key=('1' * 32).encode('ascii'))
      encryptor2 = AesEncryptor(key=('1' * 32).encode('ascii'))
      msg = b'123in0i1n 0in\x10\x00\x00\x00\x94\xd0z\xa6\xac\x95\xb2\xf5\x10\xfa\xd7\x9c\xf9\x9a\x8a\xd5 \x00\x00\x00\x89K\x9bGuxf8|\xf5\x93\xb6-\xbf\xab\xd0\x15\x99\xae}cu\xceWnE\xfdj7\xb3X_hc\xdao>0\x86\xcf\xecSI^\xe5<\xcfh'
      encrypted_msg = encryptor1.encode(msg)
      self.assertEqual(msg, encryptor2.decode(encrypted_msg))


if __name__ == '__main__':
    unittest.main()


import os
import six
import hmac
import hashlib
import struct

from generic_encoders import Encoder
from generic_encryptors.encryptors.exceptions import HmacMissMatch
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import padding


class AesEncryptor(Encoder):
  file_suffixes = ['aes']
  inputs = [six.binary_type]
  outputs = six.binary_type
  
  def __init__(self, key):
    self.key = key
    super(AesEncryptor, self)

  def _encode(self, data):
    iv = os.urandom(16)
    iv = iv
    cipher = Cipher(
      algorithms.AES(self.key),
      modes.CBC(iv),
      backend=default_backend())
    encryptor = cipher.encryptor()
    padder = padding.PKCS7(cipher.algorithm.block_size).padder()
    padded_data = padder.update(data) + padder.finalize()
    encrypted_data = encryptor.update(padded_data) + encryptor.finalize()
    hmac_data = hmac.new(encrypted_data, self.key, hashlib.sha256).digest()
    return (
        struct.pack('i', len(iv)) +
        iv +
        struct.pack('i', len(hmac_data)) +
        hmac_data +
        encrypted_data
      )

  def _decode(self, data):
    iv_length = struct.unpack('i', data[0:4])[0]
    iv = data[4 : iv_length + 4]
    hmac_length = struct.unpack('i', data[iv_length + 4: iv_length + 8])[0]
    hmac_received = data[iv_length + 8: iv_length + 8 + hmac_length]
    data = data[iv_length + 8 + hmac_length:]
    hmac_calculated = hmac.new(data, self.key, hashlib.sha256).digest()
    if not hmac.compare_digest(hmac_calculated, hmac_received):
      raise HmacMissMatch()
    cipher = Cipher(
      algorithms.AES(self.key),
      modes.CBC(iv),
      backend=default_backend())
    decryptor = cipher.decryptor()
    unpadder = padding.PKCS7(cipher.algorithm.block_size).unpadder()
    data = decryptor.update(data) + decryptor.finalize()
    return unpadder.update(data) + unpadder.finalize()
    



import os
from setuptools import setup, find_packages

dir_path = os.path.dirname(os.path.realpath(__file__))

VERSION = open(os.path.join(dir_path, 'VERSION')).read()

setup(
  name = 'generic-encryptors',
  packages = find_packages(),
  version = VERSION,
  description = '''
  A set of encryptors which provide an interface compatible with https://github.com/mmontagna/generic-encoders
  ''',
  long_description=open('README.md').read(),
  long_description_content_type='text/markdown',
  author = 'Marco Montagna',
  author_email = 'marcojoemontagna@gmail.com',
  url = 'https://github.com/mmontagna/generic-encryptors',
  keywords = ['encryptors', 'encryption', 'aes', 'decryption'],
  classifiers=(
      'Development Status :: 4 - Beta',
      'Intended Audience :: Developers',
      'Natural Language :: English',
      'License :: OSI Approved :: MIT License',
      'Programming Language :: Python',
      'Programming Language :: Python :: 2',
      'Programming Language :: Python :: 2.7',
      'Programming Language :: Python :: 3',
      'Programming Language :: Python :: 3.6',
  ),
  data_files = [('', ['LICENSE', 'VERSION'])],
  include_package_data=True,
  python_requires=">=2.7",
  license=open(os.path.join(dir_path, 'LICENSE')).read(),
  install_requires=[
    "cryptography~=2.3.0",
    "generic-encoders==0.3.2"
  ],
  entry_points = {

  },
)